package com.fjhx.account.service.account.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.account.entity.account.po.AccountDeptRunningWater;
import com.fjhx.account.entity.account.vo.AccountRequestFundsVo;
import com.fjhx.account.entity.account.vo.AccountRunningWaterVo;
import com.fjhx.account.mapper.account.AccountDeptRunningWaterMapper;
import com.fjhx.account.service.account.AccountDeptRunningWaterService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.account.entity.account.vo.AccountDeptRunningWaterVo;
import com.fjhx.account.entity.account.dto.AccountDeptRunningWaterSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.account.entity.account.dto.AccountDeptRunningWaterDto;
import cn.hutool.core.bean.BeanUtil;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 部门-资金流水关联表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-13
 */
@Service
public class AccountDeptRunningWaterServiceImpl extends ServiceImpl<AccountDeptRunningWaterMapper, AccountDeptRunningWater> implements AccountDeptRunningWaterService {

    @Autowired
    private ISysDeptService sysDeptService;

    @Override
    public Page<AccountRunningWaterVo> getPage(AccountDeptRunningWaterSelectDto dto) {
        QueryWrapper<Object> wrapper = Wrappers.query();
        wrapper.eq(ObjectUtil.isNotEmpty(dto.getStatus()), "arw.status", dto.getStatus());
        wrapper.eq(ObjectUtil.isNotEmpty(dto.getCurrency()), "arw.currency", dto.getCurrency());
        wrapper.eq(ObjectUtil.isNotEmpty(dto.getType()),"adrw.type",dto.getType());
        if (StringUtils.isNotEmpty(dto.getKeyword())) {
            wrapper.and(wrapper1 -> wrapper1.like("arw.remarks", dto.getKeyword()).or().like("arw.name", dto.getKeyword()));
        }

        Page<AccountRunningWaterVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        //切换数据源
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        //查询部门信息
        List<SysDept> sysDeptList = sysDeptService.list();
        Map<Long, List<SysDept>> sysDeptMap = sysDeptList.stream().collect(Collectors.groupingBy(SysDept::getDeptId));

        List<AccountRunningWaterVo> records = page.getRecords();
        for (AccountRunningWaterVo record : records) {
            List<SysDept> sysDepts = sysDeptMap.get(record.getDeptId());
            if (ObjectUtil.isNotEmpty(sysDepts)) {
                record.setDeptName(sysDepts.get(0).getDeptName());
            }
        }
        return page;
    }

    @Override
    public AccountDeptRunningWaterVo detail(Long id) {
        AccountDeptRunningWater AccountDeptRunningWater = this.getById(id);
        AccountDeptRunningWaterVo result = BeanUtil.toBean(AccountDeptRunningWater, AccountDeptRunningWaterVo.class);
        return result;
    }

    /**
     * 部门-资金流水关联表新增
     */
    @Override
    public void add(AccountDeptRunningWaterDto accountDeptRunningWaterDto) {
        if (ObjectUtil.isNotEmpty(accountDeptRunningWaterDto.getId())){
            this.updateById(accountDeptRunningWaterDto);
        }else {
            this.save(accountDeptRunningWaterDto);
        }

    }

    @Override
    public void edit(AccountDeptRunningWaterDto accountDeptRunningWaterDto) {
        this.updateById(accountDeptRunningWaterDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
