package com.fjhx.item.service.product.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.item.entity.product.dto.ProductInfoDto;
import com.fjhx.item.entity.product.dto.ProductInfoSelectDto;
import com.fjhx.item.entity.product.po.ProductClassify;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.entity.product.vo.ProductInfoVo;
import com.fjhx.item.mapper.product.ProductInfoMapper;
import com.fjhx.item.service.product.ProductClassifyService;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.item.util.CodeEnum;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 产品库 服务实现类
 * </p>
 *
 * @author zlj
 * @since 2023-03-17
 */
@DS(SourceConstant.ITEM)
@Service
public class ProductInfoServiceImpl extends ServiceImpl<ProductInfoMapper, ProductInfo> implements ProductInfoService {

    @Autowired
    private ProductClassifyService productClassifyService;
    @Autowired
    private ISysDeptService sysDeptService;

    @Override
    public Page<ProductInfoVo> getPage(ProductInfoSelectDto dto) {
        IWrapper<ProductInfo> wrapper = getWrapper();
        wrapper.orderByDesc("pi", ProductInfo::getId);
        wrapper.eq("pi", ProductInfo::getType, dto.getType());
        wrapper.eq("pi", ProductInfo::getDefinition, dto.getDefinition());
        wrapper.eq("pi", ProductInfo::getProductClassifyId, dto.getProductClassifyId());
        wrapper.keyword(dto,
                new SqlField("pi", ProductInfo::getName),
                new SqlField("pi", ProductInfo::getCode),
                new SqlField("pi", ProductInfo::getCustomCode)
        );
        wrapper.eq("json_unquote(victoriatourist_json ->'$.lifeCycle')",dto.getLifeCycle());
        Page<ProductInfoVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        List<ProductInfoVo> records = page.getRecords();

        if (records.size() == 0) {
            return page;
        }

        List<ProductClassify> productClassifyList = productClassifyService.list();
        Map<Long, ProductClassify> productClassifyMap = productClassifyList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));

        for (ProductInfoVo record : records) {
            Long productClassifyId = record.getProductClassifyId();
            ProductClassify productClassify = productClassifyMap.get(productClassifyId);
            if (productClassify == null) {
                continue;
            }

            record.setClassifyName(productClassify.getName());

            List<String> classifyNameGroup = new ArrayList<>();

            while (productClassify != null) {
                classifyNameGroup.add(0, productClassify.getName());
                productClassify = productClassifyMap.get(productClassify.getParentId());
            }
            record.setClassifyNameGroup(classifyNameGroup);

        }
        //赋值维多利亚扩展部门名称
        List<Long> ids = new ArrayList<>();
        for (ProductInfoVo record : records) {
            String victoriatouristJson = record.getVictoriatouristJson();
            if (ObjectUtil.isNotEmpty(victoriatouristJson)) {
                JSONObject json = JSONObject.parseObject(victoriatouristJson);
                ids.add(json.getLong("deptId"));
            }
        }
        if (ObjectUtil.isEmpty(ids)) {
            return page;
        }

        IWrapper<SysDept> wrapper1 = IWrapper.getWrapper();
        wrapper1.in(SysDept::getDeptId, ids);
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        List<SysDept> sysDepts = sysDeptService.list(wrapper1);
        DynamicDataSourceContextHolder.poll();
        Map<Long, String> sysDeptMap = sysDepts.stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        for (ProductInfoVo record : records) {
            String victoriatouristJson = record.getVictoriatouristJson();

            if (ObjectUtil.isEmpty(victoriatouristJson)) {
                continue;
            }

            JSONObject json = JSONObject.parseObject(victoriatouristJson);
            json.put("deptName", sysDeptMap.get(json.getLong("deptId")));
            record.setVictoriatouristJson(json.toJSONString());
        }

        return page;
    }

    @Override
    public ProductInfoVo detail(Long id) {
        ProductInfo ProductInfo = this.getById(id);
        ProductInfoVo result = BeanUtil.toBean(ProductInfo, ProductInfoVo.class);
        return result;
    }

    @DSTransactional
    @Override
    public void add(ProductInfoDto productInfoDto) {
        // 赋值产品编号
        productInfoDto.setCode(CodeEnum.PRODUCT.getCode());
        // 排除名称重复
        this.nameDuplication(ProductInfo::getName, productInfoDto.getName(), "产品名称重复");
        this.save(productInfoDto);
        ObsFileUtil.saveFile(productInfoDto.getFileList(), productInfoDto.getId());
        //处理维多利亚扩展
        String victoriatouristJson = productInfoDto.getVictoriatouristJson();
        if(ObjectUtil.isNotEmpty(victoriatouristJson)) {
            JSONObject json = JSONObject.parseObject(victoriatouristJson);
            json.put("deptId", SecurityUtils.getDeptId());
            productInfoDto.setVictoriatouristJson(json.toJSONString());
        }
    }

    @DSTransactional
    @Override
    public void edit(ProductInfoDto productInfoDto) {
        // 禁止产品编号修改
        productInfoDto.setCode(null);
        // 排除名称重复
        this.nameDuplication(ProductInfo::getName, productInfoDto.getName(), productInfoDto.getId(), "产品名称重复");
        this.updateById(productInfoDto);
        ObsFileUtil.editFile(productInfoDto.getFileList(), productInfoDto.getId());
    }

    @DSTransactional
    @Override
    public void delete(Long id) {
        this.removeById(id);
        ObsFileUtil.removeFile(id);
    }

    /**
     * 根据产品IDS获取产品
     *
     * @param productIds
     * @return
     */
    @Override
    public List<ProductInfoVo> getListByProductIds(List<Long> productIds) {
        return baseMapper.getListByProductIds(productIds);
    }

    /**
     * 根据产品类型获取产品
     *
     * @param productType
     * @return
     */
    @Override
    public List<ProductInfoVo> getListByProductType(Integer productType) {
        return baseMapper.getListByProductType(productType);
    }

    @Override
    public void excelImport(MultipartFile file) {
//        List<Object> read = ExcelUtil.read(file, Object.class);
    }

}
